@@ -1,5 +1,5 @@
 --
--- (C) 2019-22 - ntop.org
+-- (C) 2019-24 - ntop.org
 --
 
 -- ##############################################
@@ -9,6 +9,8 @@ local flow_alert_keys = require "flow_alert_keys"
 local classes = require "classes"
 -- Make sure to import the Superclass!
 local alert = require "alert"
+-- Import Mitre Att&ck utils
+local mitre = require "mitre_utils"
 
 -- ##############################################
 
@@ -18,9 +20,17 @@ local alert_ndpi_http_suspicious_user_agent = classes.class(alert)
 
 alert_ndpi_http_suspicious_user_agent.meta = {
    alert_key  = flow_alert_keys.flow_alert_ndpi_http_suspicious_user_agent,
-   i18n_title = "alerts_dashboard.ndpi_http_suspicious_user_agent_title",
+   i18n_title = "flow_risk.ndpi_http_suspicious_user_agent",
    icon = "fas fa-fw fa-exclamation",
-   
+
+   -- Mitre Att&ck Matrix values
+   mitre_values = {
+      mitre_tactic = mitre.tactic.c_and_c,
+      mitre_technique = mitre.technique.app_layer_proto,
+      mitre_sub_technique = mitre.sub_technique.web_proto,
+      mitre_id = "T1071.001"
+   },
+
    has_victim = true,
    has_attacker = true,
 }
